@@ -41,6 +41,7 @@
 #include "gedit-debug.h"
 #include "gedit-view.h"
 #include "gedit-mdi.h"
+#include "gedit-recent.h"
 #include "gedit2.h"
 #include <gtksourceview/gtksourceview.h>
 
@@ -81,6 +82,10 @@ static void 		gedit_prefs_manager_syntax_hl_enable_changed (GConfClient *client,
 								      guint cnxn_id, 
 								      GConfEntry *entry, 
 								      gpointer user_data);
+static void 		gedit_prefs_manager_max_recents_changed (GConfClient *client,
+								 guint cnxn_id, 
+								 GConfEntry *entry, 
+								 gpointer user_data);
 
 static gint window_state = -1;
 static gint window_height = -1;
@@ -148,6 +153,11 @@ gedit_prefs_manager_app_init (void)
 				GPM_SYNTAX_HL_ENABLE,
 				gedit_prefs_manager_syntax_hl_enable_changed,
 				NULL, NULL, NULL);
+
+		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
+				GPM_MAX_RECENTS,
+				gedit_prefs_manager_max_recents_changed,
+				NULL, NULL, NULL);
 	}
 
 	return gedit_prefs_manager != NULL;	
@@ -819,23 +829,21 @@ gedit_prefs_manager_syntax_hl_enable_changed (GConfClient *client,
 
 	if (strcmp (entry->key, GPM_SYNTAX_HL_ENABLE) == 0)
 	{
-		gboolean enable;
-			
 		GList *docs;
 		GList *l;
-		
+		gboolean enable;
+
 		if (entry->value->type == GCONF_VALUE_BOOL)
-			enable = gconf_value_get_bool (entry->value);	
+			enable = gconf_value_get_bool (entry->value);
 		else
 			enable = GPM_DEFAULT_SYNTAX_HL_ENABLE;
-	
+
 		docs = gedit_get_open_documents ();
 		l = docs;
-
 		while (l != NULL)
 		{
 			g_return_if_fail (GTK_IS_SOURCE_BUFFER (l->data));
-			
+
 			gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (l->data),
 							 enable);
 
@@ -843,6 +851,55 @@ gedit_prefs_manager_syntax_hl_enable_changed (GConfClient *client,
 		}
 
 		g_list_free (docs);
+
+		/* update the sensitivity of the Higlight Mode menu item */
+		l = gedit_get_top_windows ();
+		while (l != NULL)
+		{
+			BonoboUIComponent *ui_component;
+
+			g_return_if_fail (BONOBO_IS_WINDOW (l->data));
+
+			ui_component = bonobo_mdi_get_ui_component_from_window (l->data);
+			if (enable)
+				bonobo_ui_component_set_prop (ui_component, "/menu/View/HighlightMode",
+						      "sensitive", "1", NULL);
+			else
+				bonobo_ui_component_set_prop (ui_component, "/menu/View/HighlightMode",
+						      "sensitive", "0", NULL);
+
+			l = g_list_next (l);
+		}
+	}
+}
+
+static void
+gedit_prefs_manager_max_recents_changed (GConfClient *client,
+	guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (entry->key != NULL);
+	g_return_if_fail (entry->value != NULL);
+
+	if (strcmp (entry->key, GPM_MAX_RECENTS) == 0)
+	{
+		EggRecentModel *model;
+		gint max;
+
+		if (entry->value->type == GCONF_VALUE_INT)
+		{
+			max = gconf_value_get_int (entry->value);
+
+			if (max < 0)
+				max = GPM_DEFAULT_MAX_RECENTS;
+		}
+		else
+			max = GPM_DEFAULT_MAX_RECENTS;
+
+		model = gedit_recent_get_model ();
+
+		egg_recent_model_set_limit (model, max);
 	}
 }
 
