@@ -34,11 +34,12 @@
 #include <string.h> /* For strlen */
 
 #include <glib/gutils.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <gedit/gedit-menus.h>
 #include <gedit/gedit-plugin.h>
 #include <gedit/gedit-debug.h>
+#include <gedit/gedit-metadata-manager.h>
 
 #include <libgnomeui/gnome-stock-icons.h>
 
@@ -84,6 +85,34 @@ struct _CheckRange
 static GQuark spell_checker_id = 0;
 static GQuark check_range_id = 0;
 
+static void 
+set_spell_language_cb (GeditSpellChecker   *spell,
+		       const GeditLanguage *lang,
+		       GeditDocument 	   *doc)
+{
+	gchar *raw_uri;
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (lang != NULL);
+
+	raw_uri = gedit_document_get_raw_uri (doc);
+
+	if (raw_uri != NULL)
+	{
+		gchar *key;
+
+		key = gedit_language_to_key (lang);
+		g_return_if_fail (key != NULL);
+
+		gedit_metadata_manager_set (raw_uri, 
+					    "spell-language",
+					    key);
+
+		g_free (key);
+		g_free (raw_uri);
+	}
+}
+
 static GeditSpellChecker *
 get_spell_checker_from_document (GeditDocument *doc)
 {
@@ -98,12 +127,40 @@ get_spell_checker_from_document (GeditDocument *doc)
 
 	if (data == NULL)
 	{
+		gchar *raw_uri;
+		
 		spell = gedit_spell_checker_new ();
 
+		raw_uri = gedit_document_get_raw_uri (doc);
+
+		if (raw_uri != NULL)
+		{
+			gchar *key;
+			const GeditLanguage *lang = NULL;
+
+			key = gedit_metadata_manager_get (raw_uri, "spell-language");
+			
+			if (key != NULL)
+			{
+				lang = gedit_language_from_key (key);
+				g_free (key);
+			}
+
+			if (lang != NULL)
+				gedit_spell_checker_set_language (spell,
+								  lang,
+								  NULL);	
+		}
+
 		g_object_set_qdata_full (G_OBJECT (doc), 
 					 spell_checker_id, 
 					 spell, 
 					 (GDestroyNotify)g_object_unref);
+
+		g_signal_connect (G_OBJECT (spell),
+				  "set_language",
+				  G_CALLBACK (set_spell_language_cb),
+				  doc);
 	}
 	else
 	{
@@ -114,7 +171,7 @@ get_spell_checker_from_document (GeditDocument *doc)
 	return spell;
 }
 
-static CheckRange*
+static CheckRange *
 get_check_range (GeditDocument *doc)
 {
 	CheckRange *range;
@@ -253,9 +310,8 @@ set_check_range (GeditDocument *doc, gint start, gint end)
 	update_current (doc, start);
 }
 
-
-static gboolean
-get_current_word_extents (GeditDocument *doc, gint *start, gint *end)
+static gchar *
+get_current_word (GeditDocument *doc, gint *start, gint *end)
 {
 	const CheckRange *range;
 	
@@ -266,12 +322,12 @@ get_current_word_extents (GeditDocument *doc, gint *start, gint *end)
 
 	gedit_debug (DEBUG_PLUGINS, "");
 
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (start != NULL, FALSE);
-	g_return_val_if_fail (end != NULL, FALSE);
-	
+	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (start != NULL, NULL);
+	g_return_val_if_fail (end != NULL, NULL);
+
 	range = get_check_range (doc);
-	g_return_val_if_fail (range != NULL, FALSE);
+	g_return_val_if_fail (range != NULL, NULL);
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), 
 			&end_iter, range->end_mark);
@@ -286,7 +342,7 @@ get_current_word_extents (GeditDocument *doc, gint *start, gint *end)
 	if (!gtk_text_iter_is_end (&end_iter))
 	{
 		gedit_debug (DEBUG_PLUGINS, "Current is not end");
-		
+
 		gtk_text_iter_forward_word_end (&end_iter);
 	}
 
@@ -295,21 +351,21 @@ get_current_word_extents (GeditDocument *doc, gint *start, gint *end)
 
 	gedit_debug (DEBUG_PLUGINS, "Current word extends [%d, %d]", *start, *end);
 
-	return (*start < *end);
+	if (!(*start < *end))
+		return NULL;
+
+	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc),
+			&current_iter, &end_iter, TRUE);
 }
 
 static gboolean
 goto_next_word (GeditDocument *doc)
 {
 	CheckRange *range;
-	
 	GtkTextIter current_iter;
 	GtkTextIter old_current_iter;
-
 	GtkTextIter end_iter;
 
-	gint current;
-
 	gedit_debug (DEBUG_PLUGINS, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
@@ -319,20 +375,15 @@ goto_next_word (GeditDocument *doc)
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), 
 			&current_iter, range->current_mark);
-	
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), 
-			&end_iter, range->end_mark);
+	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
 
 	if (gtk_text_iter_compare (&current_iter, &end_iter) >= 0)
 		return FALSE;
 
 	old_current_iter = current_iter;
-	
-	gtk_text_iter_forward_word_ends (&current_iter, 2);
-	
-	gtk_text_iter_backward_word_start (&current_iter);
 
-	current = gtk_text_iter_get_offset (&current_iter);
+	gtk_text_iter_forward_word_ends (&current_iter, 2);
+	gtk_text_iter_backward_word_start (&current_iter);
 
 	if ((gtk_text_iter_compare (&old_current_iter, &current_iter) < 0) &&
 	    (gtk_text_iter_compare (&current_iter, &end_iter) < 0))
@@ -344,7 +395,7 @@ goto_next_word (GeditDocument *doc)
 		return FALSE;
 }
 
-static gchar*
+static gchar *
 get_next_mispelled_word (GeditDocument *doc)
 {
 	CheckRange *range;
@@ -360,27 +411,20 @@ get_next_mispelled_word (GeditDocument *doc)
 	spell = get_spell_checker_from_document (doc);
 	g_return_val_if_fail (spell != NULL, NULL);
 
-	if (!get_current_word_extents (doc, &start, &end))
+	word = get_current_word (doc, &start, &end);
+	if (word == NULL)
 		return NULL;
 
-	word = gedit_document_get_chars (doc, start, end);
-	g_return_val_if_fail (word != NULL, NULL);
-
 	gedit_debug (DEBUG_PLUGINS, "Word to check: %s", word);
 
 	while (gedit_spell_checker_check_word (spell, word, -1, NULL))
 	{
-		gboolean res;
-		
 		g_free (word);
 
 		if (!goto_next_word (doc))
 			return NULL;
 
-		res = get_current_word_extents (doc, &start, &end);
-		g_return_val_if_fail (res, NULL);
-
-		word = gedit_document_get_chars (doc, start, end);
+		word = get_current_word (doc, &start, &end);
 		g_return_val_if_fail (word != NULL, NULL);
 
 		gedit_debug (DEBUG_PLUGINS, "Word to check: %s", word);
@@ -392,13 +436,17 @@ get_next_mispelled_word (GeditDocument *doc)
 	if (word != NULL)
 	{
 		GtkWidget *active_view;
+		GtkTextIter s, e;
 
 		range->mw_start = start;
 		range->mw_end = end;
 
 		gedit_debug (DEBUG_PLUGINS, "Select [%d, %d]", start, end);
 
-		gedit_document_set_selection (doc, start, end);
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &s, start);
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &e, end);
+
+		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (doc), &s, &e);
 
 		active_view = gedit_get_active_view ();
 		if (active_view != NULL)
@@ -414,7 +462,9 @@ get_next_mispelled_word (GeditDocument *doc)
 }
 
 static void
-ignore_cb (GeditSpellCheckerDialog *dlg, const gchar *w, GeditDocument *doc)
+ignore_cb (GeditSpellCheckerDialog *dlg,
+	   const gchar *w,
+	   GeditDocument *doc)
 {
 	gchar *word = NULL;
 	gedit_debug (DEBUG_PLUGINS, "");
@@ -437,11 +487,15 @@ ignore_cb (GeditSpellCheckerDialog *dlg, const gchar *w, GeditDocument *doc)
 }
 
 static void
-change_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *change, GeditDocument *doc)
+change_cb (GeditSpellCheckerDialog *dlg,
+	   const gchar *word,
+	   const gchar *change,
+	   GeditDocument *doc)
 {
 	CheckRange *range;
 	gchar *w = NULL;
-	
+	GtkTextIter start, end;
+
 	gedit_debug (DEBUG_PLUGINS, "");
 	
 	g_return_if_fail (doc != NULL);
@@ -451,7 +505,13 @@ change_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *change,
 	range = get_check_range (doc);
 	g_return_if_fail (range != NULL);
 
-	w = gedit_document_get_chars (doc, range->mw_start, range->mw_end);
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start, range->mw_start);
+	if (range->mw_end < 0)
+		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
+	else
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end, range->mw_end);
+
+	w = gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start, &end, TRUE);
 	g_return_if_fail (w != NULL);
 
 	if (strcmp (w, word) != 0)
@@ -464,8 +524,8 @@ change_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *change,
        	
 	gedit_document_begin_user_action (doc);
 
-	gedit_document_delete_text (doc, range->mw_start, range->mw_end);
-	gedit_document_insert_text (doc, range->mw_start, change, -1);
+	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc), &start, &end);
+	gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &start, change, -1);
 
 	gedit_document_end_user_action (doc);
 
@@ -476,17 +536,20 @@ change_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *change,
 }
 
 static void
-change_all_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *change, GeditDocument *doc)
+change_all_cb (GeditSpellCheckerDialog *dlg,
+	       const gchar *word,
+	       const gchar *change,
+	       GeditDocument *doc)
 {
 	CheckRange *range;
 	gchar *w = NULL;
-
+	GtkTextIter start, end;
 	gchar *last_searched_text;
 	gchar *last_replaced_text;
 	gint flags = 0;
-	
+
 	gedit_debug (DEBUG_PLUGINS, "");
-	
+
 	g_return_if_fail (doc != NULL);
 	g_return_if_fail (word != NULL);
 	g_return_if_fail (change != NULL);
@@ -494,7 +557,13 @@ change_all_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *cha
 	range = get_check_range (doc);
 	g_return_if_fail (range != NULL);
 
-	w = gedit_document_get_chars (doc, range->mw_start, range->mw_end);
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start, range->mw_start);
+	if (range->mw_end < 0)
+		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
+	else
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end, range->mw_end);
+
+	w = gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start, &end, TRUE);
 	g_return_if_fail (w != NULL);
 
 	if (strcmp (w, word) != 0)
@@ -533,7 +602,6 @@ change_all_cb (GeditSpellCheckerDialog *dlg, const gchar *word, const gchar *cha
 	ignore_cb (dlg, word, doc);
 }
 
-
 static void
 add_word_cb (GeditSpellCheckerDialog *dlg, const gchar *word, GeditDocument *doc)
 {
@@ -582,7 +650,6 @@ show_no_mispelled_words_dialog (gboolean sel)
 	 gtk_widget_destroy (message_dlg);
 }
 
-
 static void
 set_language_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
@@ -606,8 +673,8 @@ spell_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GeditDocument *doc;
 	GeditSpellChecker *spell;
 	GtkWidget *dlg;
-	gint start;
-	gint end;
+	gint start, end;
+	GtkTextIter sel_start, sel_end;
 	gchar *word;
 	gboolean sel = FALSE;
 	
@@ -625,9 +692,11 @@ spell_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 		return;
 	}
 
-	if (gedit_document_get_selection (doc, &start, &end))
+	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc), &sel_start, &sel_end))
 	{
 		/* get selection points */
+		start = gtk_text_iter_get_offset (&sel_start);
+		end = gtk_text_iter_get_offset (&sel_end);
 		set_check_range (doc, start, end);
 		sel = TRUE;
 	}
@@ -660,11 +729,10 @@ spell_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	g_free (word);
 
 	gtk_widget_show (dlg);
-	
 }	
 
 static void
-auto_spell_cb (BonoboUIComponent           *ui_component,
+auto_spell_cb (BonoboUIComponent          *ui_component,
 	     const char                  *path,
 	     Bonobo_UIComponent_EventType type,
 	     const char                  *state,
@@ -825,6 +893,3 @@ init (GeditPlugin *pd)
 	return PLUGIN_OK;
 }
 
-
-
-
